package com.ether;

import com.ether.annotation.ControllerMethod;
import com.ether.annotation.ParamName;
import com.ether.user.User;
import com.ether.user.UserRole;
import com.ether.user.UserState;
import com.medias.annuaire.Annuaire;
import com.medias.annuaire.Personne;
import com.medias.mail.Mail;
import com.medias.mail.MailFactory;
import net.sf.json.JSONObject;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Required;

import javax.mail.MessagingException;
import javax.servlet.http.HttpServletRequest;
import java.security.DigestException;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author vmipsl
 * @date 12 nov 2011
 */
public class ControllerEponge
        extends ControllerEther
{
    /** *********************************************************** **/
    /** *********************** VIEWS ***************************** **/
    /** *********************************************************** **/
    @ControllerMethod(view = VIEW_ERRORS)
    public Map<String, Object> viewErrors()
            throws ServiceException
    {
        return new HashMap<String, Object>();
    }

    @ControllerMethod(view = VIEW_LOGIN)
    public Map<String, Object> viewLogin()
            throws ServiceException
    {
        return new HashMap<String, Object>();
    }

    /** *********************************************************** **/
    /** *********************** CALLS ***************************** **/
    /** *********************************************************** **/
    @ControllerMethod(requestMandatory = true, jsonResult = true)
    public JSONObject login( @ParamName(ParameterConstants.PARAMETER_LOGIN) final String login,
                             @ParamName(ParameterConstants.PARAMETER_PWD) final String password,
                             @NotNull final HttpServletRequest request )
            throws ServiceException
    {
        final JSONObject jSONPeople = new JSONObject();
        final JSONObject result = new JSONObject();

        // TODO : revoir doublon user/people, besoin du people pour les tests ds les .jsp
        if( login != null && password != null )
        {
            try
            {
                final User user = _etherService.getUserByEmail( login );
                final String encryptedPassword = EtherHelper.encryptPassword( password );
                if( user == null || ( null != user && !encryptedPassword.equals( user.getPassword() ) ) )
                    result.put( "errors", "errors.logon.notFound" );
                else if( !UserState.ACCEPTED.equals( user.getState() ) )
                    result.put( "errors", "errors.logon.notAccepted" );
                else
                {
                    final Annuaire annu = (Annuaire) getServletContext().getAttribute( "APP_ANNUAIRE" );
                    final Personne people = annu.getPersonne( login, password );
                    if( people == null )
                        result.put( "errors", "errors.logon.notFound" );
                    else
                    {
                        request.getSession().setAttribute( "SES_USER", people );
                        jSONPeople.put( "name", user.getLastName() );
                        jSONPeople.put( "firstName", user.getFirstName() );
                        jSONPeople.put( "role", user.getRole().name() );
                    }
                }
            }
            catch( Exception e )
            {
                result.put( "errors", "errors.logon.failed" );
            }
        }

        if( !jSONPeople.isEmpty() )
            result.put( "jSONPeople", jSONPeople );
        return result;
    }

    @ControllerMethod(requestMandatory = true, jsonResult = true)
    public JSONObject logout( @NotNull final HttpServletRequest request )
            throws ServiceException
    {
        request.getSession().setAttribute( "SES_USER", null );
        request.getSession().invalidate();

        return new JSONObject();
    }

    @ControllerMethod(jsonResult = true)
    public JSONObject createAccount( @NotNull @ParamName(ParameterConstants.PARAMETER_NAME) final String lastName,
                                     @Nullable @ParamName(ParameterConstants.PARAMETER_FIRST_NAME) final String firstName,
                                     @NotNull @ParamName(ParameterConstants.PARAMETER_EMAIL) final String email,
                                     @NotNull @ParamName(ParameterConstants.PARAMETER_PWD) final String password )
            throws ServiceException, WebException
    {
        try
        {
            final User existingUser = _etherService.getUserByEmail( email );
            if( null == existingUser )
            {
                final Date creationDate = new Date();

                final String encryptedPassword = EtherHelper.encryptPassword( password );
                final User user = new User( lastName, firstName, email, encryptedPassword, UserRole.COORDINATOR, UserState.WAITING, false, creationDate );

                _etherService.createUser( user );

                // Send email to administrator to inform there is a new account
                sendEmailToAdministrator( user );
            }
            else
                throw new WebException( WebException.WebCode.USER_ALREADY_EXISTS, "This email already corresponds to an User", WebException.getExceptionThrowable() );
        }
        catch( DigestException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }
        catch( NoSuchAlgorithmException e )
        {
            throw new WebException( WebException.WebCode.ERROR_ENCRYPT_PASSWORD, "This password cannot be encrypted" );
        }

        return new JSONObject();
    }

    /**
     * This method create and send an email to the administrator to inform there is a new account to validate
     *
     * @param user
     * @throws WebException
     */
    private void sendEmailToAdministrator( @NotNull final User user )
            throws WebException
    {
        try
        {
            final MailFactory mailFactory = (MailFactory) getServletContext().getAttribute( "APP_MAILFACTORY" );
            final String from = (String) getServletContext().getAttribute( "APP_WEBMASTER" );
            final String toPI = (String) getServletContext().getAttribute( "APP_PI" );
            final String subject = "[MEGAPOLI] Nouvelle demande de compte utilisateur";
            final String content = "Hello Matthias,\n\nUne nouvelle demande de compte vient d'arriver.\n\n" +
                    "   - Nom : " + user.getLastName() + '\n' +
                    "   - Prénom : " + user.getFirstName() + '\n' +
                    "   - Email : " + user.getEmail() + "\n\n" +
                    "Tu peux accepter ou refuser son inscription via le BO : http://ether.ipsl.jussieu.fr/megapoli/backoffice?methodName=view\n\n" +
                    "Bonne soirée,\nLe serveur masqué";

            final Mail mailAdministrator = new Mail( from, from, null, content, subject );
            mailFactory.sendMail( mailAdministrator );
            final Mail mailPI = new Mail( from, toPI, null, content, subject );
            mailFactory.sendMail( mailPI );
        }
        catch( MessagingException e )
        {
            throw new WebException( WebException.WebCode.ERROR_EMAIL_CANNOT_BE_SEND, "The email cannot be send to the megapoli administrator" );
        }
    }

    @Required
    public void setEtherService( final EtherService etherService )
    {
        _etherService = etherService;
    }

    private static final Log LOGGER = LogFactory.getLog( ControllerEponge.class );

    private static final String VIEW_ERRORS = "project/errors";
    private static final String VIEW_LOGIN = "project/login";

    private EtherService _etherService;
}